[ltt-dev] [PATCH 05/10 round10] add max_size arguemnt

Lai Jiangshan laijs at cn.fujitsu.com
Wed Nov 16 01:48:20 EST 2011


Signed-off-by: Lai Jiangshan <laijs at cn.fujitsu.com>
---
 rculfhash.c            |   17 +++++++++++++++++
 tests/test_urcu_hash.c |    2 +-
 urcu/rculfhash.h       |    9 ++++++---
 3 files changed, 24 insertions(+), 4 deletions(-)

diff --git a/rculfhash.c b/rculfhash.c
index 2df133c..b7be893 100644
--- a/rculfhash.c
+++ b/rculfhash.c
@@ -266,6 +266,7 @@ struct cds_lfht {
 	struct rcu_table t;
 	unsigned long min_alloc_order;
 	unsigned long min_alloc_size;
+	unsigned long max_size;
 	int flags;
 	/*
 	 * We need to put the work threads offline (QSBR) when taking this
@@ -1351,6 +1352,7 @@ void cds_lfht_create_bucket(struct cds_lfht *ht, unsigned long size)
 
 struct cds_lfht *_cds_lfht_new(unsigned long init_size,
 			unsigned long min_alloc_size,
+			unsigned long max_size,
 			int flags,
 			void (*cds_lfht_call_rcu)(struct rcu_head *head,
 					void (*func)(struct rcu_head *head)),
@@ -1369,11 +1371,22 @@ struct cds_lfht *_cds_lfht_new(unsigned long init_size,
 	/* min_alloc_size must be power of two */
 	if (!min_alloc_size || (min_alloc_size & (min_alloc_size - 1)))
 		return NULL;
+
 	/* init_size must be power of two */
 	if (!init_size || (init_size & (init_size - 1)))
 		return NULL;
+
+	if (!max_size)
+		max_size = 1UL << (MAX_TABLE_ORDER - 1);
+
+	/* max_size must be power of two */
+	if (!max_size || (max_size & (max_size - 1)))
+		return NULL;
+
 	min_alloc_size = max(min_alloc_size, MIN_TABLE_SIZE);
 	init_size = max(init_size, MIN_TABLE_SIZE);
+	max_size = max(max_size, min_alloc_size);
+	init_size = min(init_size, max_size);
 	ht = calloc(1, sizeof(struct cds_lfht));
 	assert(ht);
 	ht->flags = flags;
@@ -1393,6 +1406,7 @@ struct cds_lfht *_cds_lfht_new(unsigned long init_size,
 	ht->t.resize_target = 1UL << order;
 	ht->min_alloc_size = min_alloc_size;
 	ht->min_alloc_order = get_count_order_ulong(min_alloc_size);
+	ht->max_size = max_size;
 	cds_lfht_create_bucket(ht, 1UL << order);
 	ht->t.size = 1UL << order;
 	return ht;
@@ -1752,6 +1766,7 @@ void resize_target_update_count(struct cds_lfht *ht,
 				unsigned long count)
 {
 	count = max(count, MIN_TABLE_SIZE);
+	count = min(count, ht->max_size);
 	uatomic_set(&ht->t.resize_target, count);
 }
 
@@ -1809,6 +1824,7 @@ void cds_lfht_resize_lazy_grow(struct cds_lfht *ht, unsigned long size, int grow
 {
 	unsigned long target_size = size << growth;
 
+	target_size = min(target_size, ht->max_size);
 	if (resize_target_grow(ht, target_size) >= target_size)
 		return;
 
@@ -1827,6 +1843,7 @@ void cds_lfht_resize_lazy_count(struct cds_lfht *ht, unsigned long size,
 	if (!(ht->flags & CDS_LFHT_AUTO_RESIZE))
 		return;
 	count = max(count, MIN_TABLE_SIZE);
+	count = min(count, ht->max_size);
 	if (count == size)
 		return;		/* Already the right size, no resize needed */
 	if (count > size) {	/* lazy grow */
diff --git a/tests/test_urcu_hash.c b/tests/test_urcu_hash.c
index b9e3e81..2961177 100644
--- a/tests/test_urcu_hash.c
+++ b/tests/test_urcu_hash.c
@@ -967,7 +967,7 @@ int main(int argc, char **argv)
 	 * thread from the point of view of resize.
 	 */
 	rcu_register_thread();
-	test_ht = cds_lfht_new(init_hash_size, min_hash_alloc_size,
+	test_ht = cds_lfht_new(init_hash_size, min_hash_alloc_size, (1UL << 18),
 			(opt_auto_resize ? CDS_LFHT_AUTO_RESIZE : 0) |
 			CDS_LFHT_ACCOUNTING, NULL);
       	ret = populate_hash();
diff --git a/urcu/rculfhash.h b/urcu/rculfhash.h
index c13d3df..2eefd2c 100644
--- a/urcu/rculfhash.h
+++ b/urcu/rculfhash.h
@@ -98,6 +98,7 @@ enum {
  */
 struct cds_lfht *_cds_lfht_new(unsigned long init_size,
 			unsigned long min_alloc_size,
+			unsigned long max_size,
 			int flags,
 			void (*cds_lfht_call_rcu)(struct rcu_head *head,
 				void (*func)(struct rcu_head *head)),
@@ -112,8 +113,9 @@ struct cds_lfht *_cds_lfht_new(unsigned long init_size,
 
 /*
  * cds_lfht_new - allocate a hash table.
  * @init_size: number of nodes to allocate initially. Must be power of two.
  * @min_alloc_size: the smallest allocation size to use. Must be power of two.
  * @flags: hash table creation flags (can be combined with bitwise or: '|').
  *           0: no flags.
  *           CDS_LFHT_AUTO_RESIZE: automatically resize hash table.
@@ -134,10 +136,11 @@ struct cds_lfht *_cds_lfht_new(unsigned long init_size,
 static inline
 struct cds_lfht *cds_lfht_new(unsigned long init_size,
 			unsigned long min_alloc_size,
+			unsigned long max_size,
 			int flags,
 			pthread_attr_t *attr)
 {
-	return _cds_lfht_new(init_size, min_alloc_size, flags,
+	return _cds_lfht_new(init_size, min_alloc_size, max_size, flags,
 			call_rcu, synchronize_rcu, rcu_read_lock,
 			rcu_read_unlock, rcu_thread_offline,
 			rcu_thread_online, rcu_register_thread,
@@ -112,8 +113,9 @@ struct cds_lfht *_cds_lfht_new(unsigned long init_size,
 /*
  * cds_lfht_new - allocate a hash table.
- * @init_size: number of nodes to allocate initially. Must be power of two.
- * @min_alloc_size: the smallest allocation size to use. Must be power of two.
+ * @init_size: number of buckets to allocate initially. Must be power of two.
+ * @min_alloc_size: the smallest allocation buckets size to use. Must be power of two.
+ * @max_size: the max size of buckets for future growing. Must be power of two.
  * @flags: hash table creation flags (can be combined with bitwise or: '|').
  *           0: no flags.
-- 
1.7.4.4





More information about the lttng-dev mailing list